import React, { useEffect, useRef, useState } from 'react'
import dayjs from 'dayjs'
import {
  Box,
  CardHeader,
  CardBody,
  Button,
  Card,
  Heading,
  Container,
  VStack,
  Input,
  CloseButton,
  FormControl,
  FormLabel,
  FormHelperText,
  FormErrorMessage,
  useToast,
  useColorMode,
  useBreakpointValue,
  Flex,
  Stack
} from '@chakra-ui/react'
import { useForm, Controller } from 'react-hook-form'
import { AddIcon } from '@chakra-ui/icons'
import { useTranslation } from 'react-i18next'

import { ErrorSpan } from '../style'
import { useAppSelector } from '../../__data__/store'
import { api } from '../../__data__/api/api'
import { isTeacher } from '../../utils/user'
import { PageLoader } from '../../components/page-loader/page-loader'
import { CourseCard } from './course-card'

interface NewCourseForm {
  startDt: string
  name: string
}

export const CoursesList = () => {
  const toast = useToast()
  const user = useAppSelector((s) => s.user)
  const { data, isLoading } = api.useCoursesListQuery()
  const [createUpdateCourse, crucQuery] = api.useCreateUpdateCourseMutation()
  const [showForm, setShowForm] = useState(false)
  const toastRef = useRef(null)
  const { t } = useTranslation()

  const { colorMode } = useColorMode();
  
  // Определяем размеры для адаптивного дизайна
  const buttonSize = useBreakpointValue({ base: 'md', md: 'lg' })
  const headingSize = useBreakpointValue({ base: 'md', md: 'lg' })
  const formSpacing = useBreakpointValue({ base: 5, md: 10 })
  const containerPadding = useBreakpointValue({ base: '2', md: '4' })

  const {
    control,
    handleSubmit,
    reset,
    formState: { errors },
    getValues,
  } = useForm<NewCourseForm>({
    defaultValues: {
      startDt: dayjs().format('YYYY-MM-DD'),
      name: t('journal.pl.course.defaultName'),
    },
  })

  const onSubmit = ({ startDt, name }) => {
    toastRef.current = toast({
      title: t('journal.pl.course.sending'),
      status: 'loading',
      duration: 9000,
    })
    createUpdateCourse({ name, startDt })
  }

  useEffect(() => {
    if (crucQuery.isSuccess) {
      const values = getValues()
      if (toastRef.current) {
        toast.update(toastRef.current, {
          title: t('journal.pl.course.created'),
          description: t('journal.pl.course.successMessage', { name: values.name }),
          status: 'success',
          duration: 9000,
          isClosable: true,
        })
      }
      reset()
      setShowForm(false) // Закрываем форму после успешного создания
    }
  }, [crucQuery.isSuccess, t])

  if (isLoading) {
    return (
      <PageLoader />
    )
  }

  return (
    <Container maxW="container.xl" px={containerPadding}>
      {isTeacher(user) && (
        <Box mt={{ base: 3, md: 5 }} mb={{ base: 3, md: 5 }}>
          {showForm ? (
            <Card align="left">
              <CardHeader display="flex" flexWrap="wrap" alignItems="center">
                <Heading as="h2" size={headingSize} mt="0" flex="1" mr={2} mb={{ base: 2, md: 0 }}>
                  {t('journal.pl.course.createTitle')}
                </Heading>
                <CloseButton ml={{ base: 'auto', md: 0 }} onClick={() => setShowForm(false)} />
              </CardHeader>
              <CardBody>
                <form onSubmit={handleSubmit(onSubmit)}>
                  <VStack spacing={formSpacing} align="left">
                    <Controller
                      control={control}
                      name="startDt"
                      rules={{ required: t('journal.pl.common.requiredField') }}
                      render={({ field }) => (
                        <FormControl
                        isRequired
                        isInvalid={Boolean(errors.startDt)}
                        >
                          <FormLabel>{t('journal.pl.common.startDate')}</FormLabel>
                          <Input
                            {...field}
                            required={false}
                            placeholder={t('journal.pl.common.selectDateTime')}
                            size="md"
                            type="date"
                            />
                          {errors.startDt ? (
                            <FormErrorMessage>
                              {errors.startDt?.message}
                            </FormErrorMessage>
                          ) : (
                            <FormHelperText>
                              {t('journal.pl.course.specifyStartDate')}
                            </FormHelperText>
                          )}
                        </FormControl>
                      )}
                    />
                    <Controller
                      control={control}
                      name="name"
                      rules={{
                        required: t('journal.pl.common.requiredField'),
                      }}
                      render={({ field }) => (
                        <FormControl
                        isRequired
                        isInvalid={Boolean(errors.name)}
                        >
                          <FormLabel>{t('journal.pl.course.newLectureName')}:</FormLabel>
                          <Input
                            {...field}
                            required={false}
                            placeholder={t('journal.pl.course.namePlaceholder')}
                            size="md"
                            />
                          {errors.name && (
                            <FormErrorMessage>
                              {errors.name.message}
                            </FormErrorMessage>
                          )}
                        </FormControl>
                      )}
                      />

                    <Flex mt={formSpacing} justifyContent={{ base: 'center', md: 'flex-start' }}>
                      <Stack direction={{ base: 'column', sm: 'row' }} spacing={2} width={{ base: '100%', sm: 'auto' }}>
                        <Button
                          size={buttonSize}
                          type="submit"
                          leftIcon={<AddIcon />}
                          colorScheme="blue"
                          width={{ base: '100%', sm: 'auto' }}
                          isLoading={crucQuery.isLoading}
                        >
                          {t('journal.pl.common.create')}
                        </Button>
                        <Button
                          size={buttonSize}
                          variant="outline"
                          width={{ base: '100%', sm: 'auto' }}
                          onClick={() => setShowForm(false)}
                        >
                          {t('journal.pl.common.cancel')}
                        </Button>
                      </Stack>
                    </Flex>
                  </VStack>

                  {crucQuery?.error && (
                    <Box mt={4}>
                      <ErrorSpan>{(crucQuery?.error as any).error}</ErrorSpan>
                    </Box>
                  )}
                </form>
              </CardBody>
            </Card>
          ) : (
            <Box p={{ base: 1, md: 2 }} m={{ base: 1, md: 2 }}>
              <Button
                leftIcon={<AddIcon />}
                colorScheme="green"
                onClick={() => setShowForm(true)}
                size={buttonSize}
                width={{ base: '100%', sm: 'auto' }}
              >
                {t('journal.pl.common.add')}
              </Button>
            </Box>
          )}
        </Box>
      )}
      <VStack as="ul" align="stretch" spacing={{ base: 3, md: 4 }}>
        {data?.body?.map((c) => (
          <CourseCard
          key={c.id}
          course={c}
          />
        ))}
      </VStack>
    </Container>
  )
}
